-- Copyright (C) 2019  Qon

-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>

local M10 = 10 * 1000 * 1000

-- Entity groups
local spawner = {'biter-spawner', 'spitter-spawner'}
local tree_dying = {'tree-dying-proxy'}
local nature = {'tree-proxy', 'tile-proxy'}
function trees(name) return string.find(name, 'tree') and true or false end
function natives(name) return (string.find(name, 'biter') or string.find(name, 'spitter')) and true or false end

local absorbers = {
    {slot = 'gogreen-spawner-absorb', entities = spawner, input = false},
    {slot = 'gogreen-dying-tree-absorb', entities = tree_dying, input = false},
    {slot = 'gogreen-nature-absorb', entities = nature, input = false},
}
local killed = {
    {slot = 'gogreen-nature-killed', entities = trees, input = true},
    {slot = 'gogreen-natives-killed', entities = natives, input = true},
}


function evo_combine(a, b)
    return 1 - (1 - a) * (1 - b)
end
function getstat(flow, entities, input, slot)
    if type(entities) ~= 'table' then
        -- When entities is a function: build the enitities array dynamically by filtering the killed stat with the function
        local io_side = input and 'input_counts' or 'output_counts'
        local t = {}
        for k, v in pairs(flow[io_side]) do
            if entities(k) then table.insert(t, k) end
        end
        entities = t
    end

    local value = 0
    for _, name in ipairs(entities) do
        value = value + flow.get_flow_count{name = name, input = input, precision_index = defines.flow_precision_index.one_second, count = true}
    end
    return value
end


script.on_nth_tick(60, function(event)
    local ef = game.forces.enemy.evolution_factor

    for _, v in ipairs(absorbers) do
        ef = evo_combine(ef, getstat(game.pollution_statistics, v.entities, v.input, v.slot) * settings.global[v.slot].value / M10)
    end

    for k,f in pairs(game.forces) do
        if f ~= game.forces.enemy then
            for _, v in ipairs(killed) do
                ef = evo_combine(ef, getstat(f.kill_count_statistics, v.entities, v.input, v.slot) * settings.global[v.slot].value / M10)
            end
        end
    end

    game.forces.enemy.evolution_factor = ef
end)
